# Generated by Django 3.2.20 on 2023-07-20 15:44

from django.db import migrations
import opaque_keys.edx.django.models


class Migration(migrations.Migration):

    dependencies = [
        ('ai_aside', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='aiasidecourseenabled',
            options={},
        ),
        migrations.AlterModelOptions(
            name='aiasideunitenabled',
            options={},
        ),
        migrations.AlterField(
            model_name='aiasidecourseenabled',
            name='course_key',
            field=opaque_keys.edx.django.models.CourseKeyField(db_index=True, max_length=255, unique=True),
        ),
        migrations.AlterUniqueTogether(
            name='aiasideunitenabled',
            unique_together={('course_key', 'unit_key')},
        ),
    ]



"""Constants for AI-Aside."""

# Regex for Course ID URL patterns
COURSE_ID_REGEX = r'[^/+]+(/|\+)[^/+]+(/|\+)[^/]+'
COURSE_ID_PATTERN = r'(?P<course_id>{})'.format(COURSE_ID_REGEX)

# Regex for Usage ID URL patterns (Unit IDs)
UNIT_ID_REGEX = r'(?:i4x://?[^/]+/[^/]+/[^/]+/[^@]+(?:@[^/]+)?)|(?:[^/]+)'
UNIT_ID_PATTERN = r'(?P<unit_id>{})'.format(UNIT_ID_REGEX)



"""
Implements an API for updating unit and course settings.
"""
from ai_aside.config_api.internal import NotFoundError, _get_course, _get_course_units, _get_unit
from ai_aside.models import AIAsideCourseEnabled, AIAsideUnitEnabled
from ai_aside.waffle import summaries_configuration_enabled


def get_course_settings(course_key):
    """
    Gets the settings of a course.

    Returns: dictionary of the form:
        `{'enabled': bool}`
    """
    record = _get_course(course_key)
    fields = {
        'enabled': record.enabled
    }

    return fields


def set_course_settings(course_key, settings):
    """
    Sets the settings of a course.

    Expects: settings to be a dictionary of the form:
        `{'enabled': bool}`

    Raises NotFoundError if the settings are not found.
    """
    enabled = settings['enabled']

    if not isinstance(enabled, bool):
        raise TypeError

    update = {'enabled': enabled}

    AIAsideCourseEnabled.objects.update_or_create(
        course_key=course_key,
        defaults=update,
    )


def delete_course_settings(course_key):
    """
    Deletes the settings of a course.

    Raises NotFoundError if the settings are not found.
    """
    reset_course_unit_settings(course_key)
    record = _get_course(course_key)
    record.delete()


def get_unit_settings(course_key, unit_key):
    """
    Gets the settings of a unit.

    Returns: dictionary of the form:
        `{'enabled': bool}`
    """
    record = _get_unit(course_key, unit_key)

    fields = {
        'enabled': record.enabled
    }

    return fields


def reset_course_unit_settings(course_key):
    """
    Deletes the unit settings of a course.
    """
    return _get_course_units(course_key).delete()


def set_unit_settings(course_key, unit_key, settings):
    """
    Sets the settings of a course's unit.

    Expects: settings as a dictionary of the form:
        `{'enabled': bool}`

    Raises NotFoundError if the settings are not found.
    """
    enabled = settings['enabled']

    if not isinstance(enabled, bool):
        raise TypeError

    settings = {'enabled': enabled}

    AIAsideUnitEnabled.objects.update_or_create(
        course_key=course_key,
        unit_key=unit_key,
        defaults=settings,
    )


def delete_unit_settings(course_key, unit_key):
    """
    Deletes the settings of a unit.

    Raises NotFoundError if the settings are not found.
    """
    record = _get_unit(course_key, unit_key)
    record.delete()


def is_summary_config_enabled(course_key):
    """
    Is this course even allowed to configure summaries?

    This is just a wrapper for the waffle flag for use
    by the views.
    """
    return summaries_configuration_enabled(course_key)


def is_course_settings_present(course_key):
    """
    Exist a course for the given key?
    """
    try:
        course = _get_course(course_key)
        return course is not None
    except NotFoundError:
        return False


def is_summary_enabled(course_key, unit_key=None):
    """
    Gets the enabled state of a course's unit.
    It considers both the state of a unit's override and a course defaults.
    """

    # If the feature flag is disabled, always returns False.
    if not summaries_configuration_enabled(course_key):
        return False

    if unit_key is not None:
        try:
            unit = _get_unit(course_key, unit_key)

            if unit is not None:
                return unit.enabled
        except NotFoundError:
            pass

    try:
        course = _get_course(course_key)
    except NotFoundError:
        return False

    if course is not None:
        return course.enabled

    return False



"""
ai_aside common settings.
"""


def plugin_settings(settings):
    """
    Get aside settings from calling application
    """
    env_tokens = getattr(settings, 'ENV_TOKENS', {})
    settings.SUMMARY_HOOK_HOST = env_tokens.get('SUMMARY_HOOK_HOST', '')
    settings.SUMMARY_HOOK_JS_PATH = env_tokens.get('SUMMARY_HOOK_JS_PATH', '')
    settings.AISPOT_LMS_NAME = env_tokens.get('AISPOT_LMS_NAME', '')
    settings.HTML_TAGS_TO_REMOVE = ['script', 'style']



"""
AI Aside block models.
"""

from django.db import models
from opaque_keys.edx.django.models import CourseKeyField, UsageKeyField


class AIAsideCourseEnabled(models.Model):
    """
    Maps Course Key to enabled boolean.
    """

    course_key = CourseKeyField(db_index=True, max_length=255, unique=True)
    enabled = models.BooleanField(default=False, null=False)

    created = models.DateTimeField(auto_now_add=True, db_index=True)
    modified = models.DateTimeField(auto_now=True)

    def __str__(self):
        """Query."""
        return (
            "id={id} "
            "created={created} "
            "course_key={course_key} "
            "enabled={enabled}".format(
                id=self.id,
                created=self.created.isoformat(),
                course_key=self.course_key,
                enabled=self.enabled,
            )
        )


class AIAsideUnitEnabled(models.Model):
    """
    Maps a Unit Key to enabled boolean.
    """

    course_key = CourseKeyField(db_index=True, max_length=255)
    unit_key = UsageKeyField(db_index=True, max_length=255)
    enabled = models.BooleanField(default=False, null=False)

    created = models.DateTimeField(auto_now_add=True, db_index=True)
    modified = models.DateTimeField(auto_now=True)

    class Meta:
        """Course and unit are unique together."""

        unique_together = ('course_key', 'unit_key')

    def __str__(self):
        """Query."""
        return (
            "id={id} "
            "created={created} "
            "course_key={course_key} "
            "unit_key={unit_key} "
            "enabled={enabled}".format(
                id=self.id,
                created=self.created.isoformat(),
                course_key=self.course_key,
                unit_key=self.unit_key,
                enabled=self.enabled,
            )
        )



"""Xblock aside enabling OpenAI driven summaries."""

import logging
from datetime import datetime

from django.conf import settings
from django.template import Context, Template
from web_fragments.fragment import Fragment
from webob import Response
from xblock.core import XBlock, XBlockAside

from ai_aside.config_api.api import is_summary_enabled
from ai_aside.platform_imports import get_block, get_text_transcript
from ai_aside.text_utils import html_to_text
from ai_aside.waffle import summaries_configuration_enabled as ff_is_summary_config_enabled
from ai_aside.waffle import summary_enabled as ff_summary_enabled
from ai_aside.waffle import summary_staff_only as ff_summary_staff_only

log = logging.getLogger(__name__)

# map block types to what ai-spot expects for content types
CATEGORY_TYPE_MAP = {
    "html": "TEXT",
    "video": "VIDEO",
}

summary_fragment = """
<div>&nbsp;</div>
<div class="summary-hook">
  <div summary-launch>
    <div id="launch-summary-button"
      data-url-api="{{data_url_api}}"
      data-course-id="{{data_course_id}}"
      data-content-id="{{data_content_id}}"
      data-handler-url="{{data_handler_url}}"
    >
    </div>
  </div>
  <div id="ai-spot-root"></div>
  <script type="text/javascript" src="{{js_url}}" defer="defer"></script>
</div>
"""


def _format_date(date):
    return date.strftime('%Y-%m-%d %H:%M:%S') if isinstance(date, datetime) else None


def _staff_user(block):
    return getattr(block.runtime, 'user_is_staff', False)


def _render_summary(context):
    template = Template(summary_fragment)
    return template.render(Context(context))


def _extract_child_contents(child, category):
    """
    Process the child contents based on its category.

    Returns a string or None if there are no contents available.
    """
    if category == 'html':
        content_html = child.get_html()
        text = html_to_text(content_html)

        return text

    if category == 'video':
        transcript = get_text_transcript(child)  # may be None
        return transcript

    return None


def _parse_children_contents(block):
    """
    Extract the analyzable contents from block children.

    Returns length and an item list.
    """
    if not _check_summarizable(block):
        return 0, []

    content_items = []

    children = block.get_children()
    content_length = 0
    for child in children:
        category = getattr(child, 'category', None)
        category_type = CATEGORY_TYPE_MAP.get(category)
        published_on = getattr(child, 'published_on', None)
        edited_on = getattr(child, 'edited_on', None)
        definition_id = str(getattr(getattr(child, 'scope_ids', None), 'def_id', None))
        text = _extract_child_contents(child, category)

        if text is None:
            continue

        content_length += len(text)
        content_items.append({
            'definition_id': definition_id,
            'content_type': category_type,
            'content_text': text,
            'published_on': published_on,
            'edited_on': edited_on,
        })

    return content_length, content_items


def _check_summarizable(block):
    """
    First pass check if a block has or does not have sufficient text to summarize.

    We don't sanitize the content due to performance in this first check.
    """
    children = block.get_children()

    content_length = 0

    for child in children:
        category = child.category
        if category == 'html':
            content_length += len(child.get_html())
            if content_length > settings.SUMMARY_HOOK_MIN_SIZE:
                return True

        if category == 'video':
            return True

    return False


class SummaryHookAside(XBlockAside):
    """
    XBlock aside that injects AI summary javascript.
    """

    @XBlock.handler
    def summary_handler(self, request=None, suffix=None):  # pylint: disable=unused-argument
        """
        Extract and return summarizable text from unit children.

        Only services and staff users are allowed to fetch summary text, everyone else
        gets an unhelpful 403.
        """
        if not _staff_user(self):
            return Response(status=403)

        block = get_block(self.scope_ids.usage_id.usage_key)
        valid = self.should_apply_to_block(block)

        if not valid:
            return Response(status=404)

        published_on = getattr(block, 'published_on', None)
        edited_on = getattr(block, 'published_on', None)

        data = []

        length, items = _parse_children_contents(block)

        if length < settings.SUMMARY_HOOK_MIN_SIZE or len(items) < 1:
            return Response(json_body={'data': []})

        for item in items:
            data.append({
                **item,
                'published_on': _format_date(item['published_on']),
                'edited_on': _format_date(item['edited_on']),
            })

        json = {
            'content_id': str(block.scope_ids.usage_id),
            'course_id': str(block.scope_ids.usage_id.course_key),
            'data': data,
            'published_on': _format_date(published_on),
            'edited_on': _format_date(edited_on),
        }
        return Response(json_body=json)

    @XBlockAside.aside_for('student_view')
    def student_view_aside(self, block, context=None):  # pylint: disable=unused-argument
        """
        Render the aside contents for the student view.

        Returns a Fragment.

        This function absorbs all exceptions to protect the LMS,
        delegating real work to _student_view_can_throw
        """
        try:
            return self._student_view_can_throw(block)
        except Exception as ex:  # pylint: disable=broad-exception-caught
            usage_id = block.scope_ids.usage_id
            log.error(f'Summary hook aside suppressed exception on {usage_id} during student_view_aside: {ex}')
            return Fragment('')

    def _student_view_can_throw(self, block):
        """
        Render the aside contents for the student view.

        Returns a Fragment.

        This function can throw exceptions.
        """
        fragment = Fragment('')

        # Check if there is content that worths summarizing
        length, _ = _parse_children_contents(block)
        if length < settings.SUMMARY_HOOK_MIN_SIZE:
            return fragment

        usage_id = block.scope_ids.usage_id

        log.info(f'Summary hook injecting into {usage_id}')

        handler_url = self._summary_handler_url()

        fragment.add_content(
            _render_summary(
                {
                    'data_url_api': settings.SUMMARY_HOOK_HOST,
                    'data_course_id': usage_id.course_key,
                    'data_content_id': usage_id,
                    'data_handler_url': handler_url,
                    'js_url': settings.SUMMARY_HOOK_HOST + settings.SUMMARY_HOOK_JS_PATH,
                }
            )
        )
        return fragment

    def _summary_handler_url(self):
        """
        Generate the summary handler URL for this block.

        A separate function to handle overrides required
        for the unusual use of the handler (non-edx codebase edx service)
        and to override the URL for use in devstack.
        """
        # thirdparty=true gives the full host name and unauthenticated handler
        handler_url = self.runtime.handler_url(self, 'summary_handler', thirdparty=True)
        # but we want the authenticated handler
        handler_url = handler_url.replace('handler_noauth', 'handler')
        # enable ai-spot to see the LMS when they are installed together in devstack
        aispot_lms_name = settings.AISPOT_LMS_NAME
        if aispot_lms_name != '':
            handler_url = handler_url.replace('localhost', aispot_lms_name)
        return handler_url

    @classmethod
    def should_apply_to_block(cls, block):
        """
        Determine whether this aside should apply to a given block type, course, and user.

        This function absorbs all exceptions to protect the LMS,
        delegating real work to _should_apply_throws.
        """
        try:
            return cls._should_apply_can_throw(block)
        except Exception as ex:  # pylint: disable=broad-exception-caught
            usage_id = block.scope_ids.usage_id
            log.error(f'Summary hook aside suppressed exception on {usage_id} during should_apply_to_block: {ex}')
            return False

    @classmethod
    def _should_apply_can_throw(cls, block):
        """
        Determine whether this aside should apply to a given block type, course, and user.

        This function can throw exceptions.
        """
        if getattr(block, 'category', None) != 'vertical':
            return False

        course_key = block.scope_ids.usage_id.course_key
        unit_key = block.scope_ids.usage_id

        if _staff_user(block) and ff_summary_staff_only(course_key):
            return True

        if ff_is_summary_config_enabled(course_key):
            return is_summary_enabled(course_key, unit_key)

        if ff_summary_enabled(course_key):
            return True

        return False



"""
URLs for ai_aside.
"""
from django.urls import path, re_path  # pylint: disable=unused-import
from django.views.generic import TemplateView  # pylint: disable=unused-import

from ai_aside.config_api.urls import urlpatterns as apipatterns

app_name = 'ai_aside'

urlpatterns = []

urlpatterns += apipatterns



"""
Text manipulation utils.
"""

from html.parser import HTMLParser
from re import sub

from django.conf import settings


def cleanup_text(text):
    """
    Remove litter from replacing or manipulating text.
    """
    stripped = sub(r'[^\S\r\n]+', ' ', text)  # Removing extra spaces
    stripped = sub(r'\n{2,}', '\n', stripped)  # Removing extra new lines
    stripped = sub(r'(\s+)?\n(\s+)?', '\n', stripped)  # Removing starting extra spacesbetween new lines
    stripped = sub(r'(^(\s+)\n?)|(\n(\s+)?$)', '', stripped)  # Trim

    return stripped


class _HTMLToTextHelper(HTMLParser):  # lint-amnesty, pylint: disable=abstract-method
    """
    Helper function for html_to_text below.
    """

    _is_content = True

    def __init__(self):
        HTMLParser.__init__(self)
        self.reset()
        self.fed = []

    def handle_starttag(self, tag, _):
        """On each tag, check whether this is a tag we think is content."""
        tags_to_filter = getattr(settings, 'HTML_TAGS_TO_REMOVE', None)
        self._is_content = not (tags_to_filter and tag in tags_to_filter)

    def handle_data(self, data):
        """Handle tag data by appending text we think is content."""
        if self._is_content:
            self.fed.append(data)

    def handle_entityref(self, name):
        """If there is an entity, append the reference to the text."""
        if self._is_content:
            self.fed.append('&%s;' % name)

    def get_data(self):
        """Join together the separate data chunks into one cohesive string."""
        return ''.join(self.fed)


def html_to_text(html):
    """Strip the html tags off of the text to return plaintext."""
    htmlstripper = _HTMLToTextHelper()
    htmlstripper.feed(html)
    text = htmlstripper.get_data()
    text = cleanup_text(text)

    return text



"""
implements a simple REST API for updating unit and course settings
"""
from django.urls import re_path

from ai_aside.config_api.views import CourseEnabledAPIView, CourseSummaryConfigEnabledAPIView, UnitEnabledAPIView
from ai_aside.constants import COURSE_ID_PATTERN, UNIT_ID_PATTERN

urlpatterns = [
    re_path(r'^v1/{course_id}/?$'.format(
        course_id=COURSE_ID_PATTERN
    ), CourseEnabledAPIView.as_view(), name='api-course-settings'),
    re_path(r'^v1/{course_id}/configurable/?$'.format(
        course_id=COURSE_ID_PATTERN
    ), CourseSummaryConfigEnabledAPIView.as_view(), name='api-course-configurable'),
    re_path(r'^v1/{course_id}/{unit_id}/?$'.format(
        course_id=COURSE_ID_PATTERN,
        unit_id=UNIT_ID_PATTERN
    ), UnitEnabledAPIView.as_view(), name='api-unit-settings'),
]



# Generated by Django 3.2.19 on 2023-07-06 17:44

from django.db import migrations, models
import opaque_keys.edx.django.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AIAsideCourseEnabled',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('course_key', opaque_keys.edx.django.models.CourseKeyField(db_index=True, max_length=255)),
                ('enabled', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('modified', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
        migrations.CreateModel(
            name='AIAsideUnitEnabled',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('course_key', opaque_keys.edx.django.models.CourseKeyField(db_index=True, max_length=255)),
                ('unit_key', opaque_keys.edx.django.models.UsageKeyField(db_index=True, max_length=255)),
                ('enabled', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('modified', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
    ]



"""Tests for the block."""
import unittest
from datetime import datetime
from textwrap import dedent
from unittest.mock import MagicMock, patch

from django.test import TestCase, override_settings

from ai_aside.block import _check_summarizable, _extract_child_contents, _format_date, _parse_children_contents

fake_transcript = 'This is the text version from the transcript'


def fake_get_transcript(child, lang=None, output_format='SRT', youtube_id=None):  # pylint: disable=unused-argument
    return (fake_transcript, 'unused', 'unused')


class FakeChild:
    """Fake child block for testing"""
    transcript_download_format = 'txt'

    def __init__(self, category, test_id='test-id', test_html='<div>This is a test</div>'):
        self.category = category
        self.published_on = 'published-on-{}'.format(test_id)
        self.edited_on = 'edited-on-{}'.format(test_id)
        self.scope_ids = lambda: None
        self.scope_ids.def_id = 'def-id-{}'.format(test_id)
        self.html = test_html
        self.transcript = fake_transcript

    def get_html(self):
        if self.category == 'html':
            return self.html

        return None


class FakeBlock:
    "Fake block for testing, returns given children"
    def __init__(self, children):
        self.children = children

    def get_children(self):
        return self.children


@override_settings(SUMMARY_HOOK_MIN_SIZE=40, HTML_TAGS_TO_REMOVE=['script', 'style', 'test'])
class TestSummaryHookAside(TestCase):
    """Summary hook aside tests"""
    def setUp(self):
        transcript_utils_mock = MagicMock()
        transcript_utils_mock.get_transcript = fake_get_transcript

        xmodule_exceptions_mock = MagicMock()
        xmodule_exceptions_mock.NotFoundError = MagicMock()

        modules = {
            'xmodule.exceptions': xmodule_exceptions_mock,
            'xmodule.video_block.transcripts_utils': transcript_utils_mock,
        }

        patch.dict('sys.modules', modules).start()

    def test_format_date(self):
        date = datetime(2023, 5, 1, 12, 0, 0)
        formatted_date = _format_date(date)
        self.assertEqual(formatted_date, '2023-05-01 12:00:00')

    def test_format_date_with_invalid_input(self):
        invalid_date = '2023-05-01'
        formatted_date = _format_date(invalid_date)
        self.assertIsNone(formatted_date)

    def test_extract_child_contents_with_html(self):
        category = 'html'

        child = FakeChild(category)
        content = _extract_child_contents(child, category)

        self.assertEqual(content, 'This is a test')

    def test_extract_child_contents_with_video(self):
        category = 'video'

        child = FakeChild(category)
        content = _extract_child_contents(child, category)
        self.assertEqual(content, 'This is the text version from the transcript')

    def test_extract_child_contents_with_invalid_category(self):
        category = 'invalid'

        child = FakeChild(category)
        content = _extract_child_contents(child, category)
        self.assertIsNone(content)

    def test_check_summarizable_with_valid_children(self):
        children = [
            FakeChild('html', '01', '''
                <p>
                    Lorem ipsum dolor sit amet, consectetur adipiscing elit.
                    Vivamus dapibus elit lacus, at vehicula arcu vehicula in.
                    In id felis arcu. Maecenas elit quam, volutpat cursus pharetra vel, tempor at lorem.
                    Fusce luctus orci quis tempor aliquet.
                </p>'''),
            FakeChild('html', '02', '''
                <Everything on the content on this child is inside a tag, so parsing it would return almost>
                    Nothing
                </The quick brown fox jumps over the lazy dog>'''),
            FakeChild('video', '03'),
            FakeChild('unknown', '04'),
        ]
        block = FakeBlock(children)

        content = _check_summarizable(block)

        self.assertTrue(content)

    def test_check_summarizable_with_invalid_children(self):
        children = [
            FakeChild('html', '01', '<p>Test</p>'),
            FakeChild('unknown', '04'),
        ]
        block = FakeBlock(children)

        content = _check_summarizable(block)

        self.assertFalse(content)

    def test_parse_children_contents_with_valid_children(self):
        children = [
            FakeChild('html', '01', '''
                <p>
                    Lorem ipsum dolor sit amet, consectetur adipiscing elit.
                    Vivamus dapibus elit lacus, at vehicula arcu vehicula in.
                    In id felis arcu. Maecenas elit quam, volutpat cursus pharetra vel, tempor at lorem.
                    Fusce luctus orci quis tempor aliquet.
                </p>'''),
            FakeChild('html', '02', '''
                <Everything on the content on this child is inside a tag, so parsing it would return almost>
                    Nothing
                </The quick brown fox jumps over the lazy dog>'''),
            FakeChild('video', '03'),
            FakeChild('unknown', '04'),
        ]
        block = FakeBlock(children)

        expected_length, expected_items = (
            289,
            [{
                'definition_id': 'def-id-01',
                'content_type': 'TEXT',
                'content_text': dedent('''\
                    Lorem ipsum dolor sit amet, consectetur adipiscing elit.
                    Vivamus dapibus elit lacus, at vehicula arcu vehicula in.
                    In id felis arcu. Maecenas elit quam, volutpat cursus pharetra vel, tempor at lorem.
                    Fusce luctus orci quis tempor aliquet.'''),
                'published_on': 'published-on-01',
                'edited_on': 'edited-on-01',
            }, {
                'definition_id': 'def-id-02',
                'content_type': 'TEXT',
                'content_text': 'Nothing',
                'published_on': 'published-on-02',
                'edited_on': 'edited-on-02',
            }, {
                'definition_id': 'def-id-03',
                'content_type': 'VIDEO',
                'content_text': fake_transcript,
                'published_on': 'published-on-03',
                'edited_on': 'edited-on-03',
            }]
        )

        length, items = _parse_children_contents(block)

        self.assertEqual(length, expected_length)
        self.assertEqual(items, expected_items)

    def test_parse_children_contents_with_valid_children_2(self):
        children = [
            FakeChild('html', '01', '''
                <p>
                    Lorem ipsum.
                </p>'''),
            FakeChild('html', '02', '''
                <Everything on the content on this child is inside a tag, so parsing it would return almost>
                    Nothing
                </The quick brown fox jumps over the lazy dog>'''),
            FakeChild('unknown', '03'),
        ]
        block = FakeBlock(children)

        expected_length, expected_items = (
            19,
            [{
                'definition_id': 'def-id-01',
                'content_type': 'TEXT',
                'content_text': 'Lorem ipsum.',
                'published_on': 'published-on-01',
                'edited_on': 'edited-on-01',
            }, {
                'definition_id': 'def-id-02',
                'content_type': 'TEXT',
                'content_text': 'Nothing',
                'published_on': 'published-on-02',
                'edited_on': 'edited-on-02',
            }]
        )

        length, items = _parse_children_contents(block)
        self.assertEqual(length, expected_length)
        self.assertEqual(items, expected_items)

    def test_parse_children_contents_with_script_or_style_tags(self):
        children = [
            FakeChild('html', '01', '''
                <span>This should be the only text to be extracted.</span>
                <test>For testing purposes only, this tag is ignored as well</test>
                <script>
                  function ignore() {
                    console.log('This content should be ignored.');
                  }
                </script>
                <script type="text/javascript">
                  console.log('This should be ignored as well.')
                </script>
                <script src="https://nevermind.me/i-should-also-be-discarded.js" type="text/javascript"></script>'''),
            FakeChild('html', '02', '''
                <div class="cypher">Why oh why didn't I take the <em>BLUE</em> pill?</div>
                <style>
                  .cypher em {
                    color: #00f;
                  }
                </style>'''),
        ]
        block = FakeBlock(children)

        expected_length, expected_items = (
            84,
            [{
                'definition_id': 'def-id-01',
                'content_type': 'TEXT',
                'content_text': 'This should be the only text to be extracted.',
                'published_on': 'published-on-01',
                'edited_on': 'edited-on-01',
            }, {
                'definition_id': 'def-id-02',
                'content_type': 'TEXT',
                'content_text': 'Why oh why didn\'t I take the BLUE pill?',
                'published_on': 'published-on-02',
                'edited_on': 'edited-on-02',
            }]
        )

        length, items = _parse_children_contents(block)

        self.assertEqual(length, expected_length)
        self.assertEqual(items, expected_items)

    def test_parse_children_contents_with_invalid_children(self):
        children = [
            FakeChild('html', '01', '<div>This</div>'),
            FakeChild('unknown', '03', '<div>This will not be parsed, category: <em>unknown</em>.<p>'),
        ]
        block = FakeBlock(children)

        length, items = _parse_children_contents(block)

        self.assertEqual(length, 0)
        self.assertEqual(items, [])


@override_settings(SUMMARY_HOOK_MIN_SIZE=40, HTML_TAGS_TO_REMOVE=['script', 'style', 'test'])
class TestSummaryHookAsideMissingTranscript(TestCase):
    """Summary hook aside tests for the case where a transcript is missing"""
    def setUp(self):
        class FakeNotFoundError(BaseException):
            pass

        xmodule_exceptions_mock = MagicMock()
        xmodule_exceptions_mock.NotFoundError = FakeNotFoundError

        def error_get_transcript(child, lang=None, output_format='SRT',
                                 youtube_id=None):
            raise FakeNotFoundError()

        transcript_utils_mock = MagicMock()
        transcript_utils_mock.get_transcript = error_get_transcript

        modules = {
            'xmodule.exceptions': xmodule_exceptions_mock,
            'xmodule.video_block.transcripts_utils': transcript_utils_mock,
        }

        patch.dict('sys.modules', modules).start()

    def test_extract_child_contents_with_broken_video(self):
        category = 'video'

        child = FakeChild(category)
        content = _extract_child_contents(child, category)
        self.assertEqual(content, None)

    def test_parse_children_contents_with_partly_valid_children(self):
        children = [
            FakeChild('html', '01', '''
                <p>
                    Lorem ipsum dolor sit amet, consectetur adipiscing elit.
                    Vivamus dapibus elit lacus, at vehicula arcu vehicula in.
                    In id felis arcu. Maecenas elit quam, volutpat cursus pharetra vel, tempor at lorem.
                    Fusce luctus orci quis tempor aliquet.
                </p>'''),
            FakeChild('html', '02', '''
                <Everything on the content on this child is inside a tag, so parsing it would return almost>
                    Nothing
                </The quick brown fox jumps over the lazy dog>'''),
            FakeChild('video', '03'),
            FakeChild('unknown', '04'),
        ]
        block = FakeBlock(children)

        expected_length, expected_items = (
            245,
            [{
                'definition_id': 'def-id-01',
                'content_type': 'TEXT',
                'content_text': dedent('''\
                    Lorem ipsum dolor sit amet, consectetur adipiscing elit.
                    Vivamus dapibus elit lacus, at vehicula arcu vehicula in.
                    In id felis arcu. Maecenas elit quam, volutpat cursus pharetra vel, tempor at lorem.
                    Fusce luctus orci quis tempor aliquet.'''),
                'published_on': 'published-on-01',
                'edited_on': 'edited-on-01',
            }, {
                'definition_id': 'def-id-02',
                'content_type': 'TEXT',
                'content_text': 'Nothing',
                'published_on': 'published-on-02',
                'edited_on': 'edited-on-02',
            }]
        )

        length, items = _parse_children_contents(block)

        self.assertEqual(length, expected_length)
        self.assertEqual(items, expected_items)


if __name__ == '__main__':
    unittest.main()



"""
Internal methods for the API.
"""
from ai_aside.models import AIAsideCourseEnabled, AIAsideUnitEnabled


class NotFoundError(Exception):
    "Raised when the course/unit is not found in the database"


def _get_course(course_key):
    "Private method that gets a course based on an id"
    try:
        record = AIAsideCourseEnabled.objects.get(
            course_key=course_key,
        )
    except AIAsideCourseEnabled.DoesNotExist as exc:
        raise NotFoundError from exc

    return record


def _get_unit(course_key, unit_key):
    "Private method that gets a unit based on an id"
    try:
        record = AIAsideUnitEnabled.objects.get(
            course_key=course_key,
            unit_key=unit_key,
        )
    except AIAsideUnitEnabled.DoesNotExist as exc:
        raise NotFoundError from exc

    return record


def _get_course_units(course_key):
    "Private method that gets a unit based on a course_key"
    return AIAsideUnitEnabled.objects.filter(
        course_key=course_key,
    )



#!/usr/bin/env python
"""
Package metadata for ai_aside.
"""
import os
import re
import sys

from setuptools import find_packages, setup


def get_version(*file_paths):
    """
    Extract the version string from the file.

    Input:
     - file_paths: relative path fragments to file with
                   version string
    """
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename, encoding="utf8").read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.

    Requirements will include any constraints from files specified
    with -c in the requirements files.
    Returns a list of requirement strings.
    """
    requirements = {}
    constraint_files = set()

    # groups "pkg<=x.y.z,..." into ("pkg", "<=x.y.z,...")
    requirement_line_regex = re.compile(r"([a-zA-Z0-9-_.]+)([<>=][^#\s]+)?")

    def add_version_constraint_or_raise(current_line, current_requirements, add_if_not_present):
        regex_match = requirement_line_regex.match(current_line)
        if regex_match:
            package = regex_match.group(1)
            version_constraints = regex_match.group(2)
            existing_version_constraints = current_requirements.get(package, None)
            # fine to add constraints to an unconstrained package,
            # raise an error if there are already constraints in place
            if existing_version_constraints and existing_version_constraints != version_constraints:
                raise BaseException(f'Multiple constraint definitions found for {package}:'
                                    f' "{existing_version_constraints}" and "{version_constraints}".'
                                    f'Combine constraints into one location with {package}'
                                    f'{existing_version_constraints},{version_constraints}.')
            if add_if_not_present or package in current_requirements:
                current_requirements[package] = version_constraints

    # read requirements from .in
    # store the path to any constraint files that are pulled in
    for path in requirements_paths:
        with open(path) as reqs:
            for line in reqs:
                if is_requirement(line):
                    add_version_constraint_or_raise(line, requirements, True)
                if line and line.startswith('-c') and not line.startswith('-c http'):
                    constraint_files.add(os.path.dirname(path) + '/' + line.split('#')[0].replace('-c', '').strip())

    # process constraint files: add constraints to existing requirements
    for constraint_file in constraint_files:
        with open(constraint_file) as reader:
            for line in reader:
                if is_requirement(line):
                    add_version_constraint_or_raise(line, requirements, False)

    # process back into list of pkg><=constraints strings
    constrained_requirements = [f'{pkg}{version or ""}' for (pkg, version) in sorted(requirements.items())]
    return constrained_requirements


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement.

    Returns:
        bool: True if the line is not blank, a comment,
        a URL, or an included file
    """
    return line and line.strip() and not line.startswith(("-r", "#", "-e", "git+", "-c"))


VERSION = get_version('ai_aside', '__init__.py')

if sys.argv[-1] == 'tag':
    print("Tagging the version on github:")
    os.system("git tag -a %s -m 'version %s'" % (VERSION, VERSION))
    os.system("git push --tags")
    sys.exit()

README = open(os.path.join(os.path.dirname(__file__), 'README.rst'), encoding="utf8").read()
CHANGELOG = open(os.path.join(os.path.dirname(__file__), 'CHANGELOG.rst'), encoding="utf8").read()

setup(
    name='ai-aside',
    version=VERSION,
    description="""A plugin containing xblocks and apps supporting GPT and other LLM use on edX.""",
    long_description=README + '\n\n' + CHANGELOG,
    author='edX',
    author_email='ashultz@edx.org',
    url='https://github.com/openedx/ai-aside',
    packages=find_packages(
        include=['ai_aside', 'ai_aside.*'],
        exclude=["*tests"],
    ),

    include_package_data=True,
    install_requires=load_requirements('requirements/base.in'),
    python_requires=">=3.8",
    license="AGPL 3.0",
    zip_safe=False,
    keywords='Python edx',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
    entry_points={
        "lms.djangoapp": [
            "ai_aside = ai_aside.apps:AiAsideConfig",
        ],
        "cms.djangoapp": [
            "ai_aside = ai_aside.apps:AiAsideConfig",
        ],
        "xblock_asides.v1": [
            "summaryhook_aside = ai_aside.block:SummaryHookAside",
        ],
    },

)



"""
Implements a simple REST API for updating unit and course settings.

Setters:
    POST: ai_aside/v1/:course_id - (payload: { enabled: True/False })
    POST: ai_aside/v1/:course_id/:unit_id - (payload: { enabled: True/False })

Getters:
    GET: ai_aside/v1/:course_id - (response: { success: True/False, enabled: True/False })
    GET: ai_aside/v1/:course_id/:unit_id - (response: { success: True/False, enabled: True/False })

Delete:
    DELETE: ai_aside/v1/:course_id - (response: { success: True/False })
    DELETE: ai_aside/v1/:course_id/:unit_id - (response: { success: True/False })

Both GET and DELETE methods respond with a 404 if the setting cannot be found.
"""
from opaque_keys import InvalidKeyError
from opaque_keys.edx.keys import CourseKey, UsageKey
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from ai_aside.config_api.api import (
    NotFoundError,
    delete_course_settings,
    delete_unit_settings,
    get_course_settings,
    get_unit_settings,
    is_summary_config_enabled,
    reset_course_unit_settings,
    set_course_settings,
    set_unit_settings,
)


class APIResponse(Response):
    """API Response"""
    def __init__(self, data=None, http_status=None, content_type=None, success=False):
        _status = http_status or status.HTTP_200_OK
        data = data or {}
        reply = {'response': {'success': success}}
        reply['response'].update(data)
        super().__init__(data=reply, status=_status, content_type=content_type)


class CourseSummaryConfigEnabledAPIView(APIView):
    """
    Simple GET endpoint to expose whether the course may use summary config.
    """

    def get(self, request, course_id=None):
        """Expose whether the course may use summary config"""
        if course_id is None:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        try:
            enabled = is_summary_config_enabled(CourseKey.from_string(course_id))
            return APIResponse(success=True, data={'enabled': enabled})
        except InvalidKeyError:
            data = {'message': 'Invalid Key'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)


class CourseEnabledAPIView(APIView):
    """Handlers for course level settings"""

    def get(self, request, course_id=None):
        """Gets the enabled state for a course"""
        if course_id is None:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        try:
            settings = get_course_settings(CourseKey.from_string(course_id))
        except InvalidKeyError:
            data = {'message': 'Invalid Key'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)
        except NotFoundError:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        return APIResponse(success=True, data=settings)

    def post(self, request, course_id=None):
        """Update the course and reset if its necessary"""

        # enabled: Updates the course enabled default state
        enabled = request.data.get('enabled')

        # reset: If it is present, it will delete all unit settings, resetting them back to the default
        reset = request.data.get('reset')

        try:
            course_key = CourseKey.from_string(course_id)
            set_course_settings(course_key, {'enabled': enabled})
            if reset:
                reset_course_unit_settings(course_key)
        except InvalidKeyError:
            data = {'message': 'Invalid Key'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)
        except TypeError:
            data = {'message': 'Invalid parameters'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)

        return APIResponse(success=True)

    def delete(self, request, course_id=None):
        """Deletes the settings for a module"""

        if course_id is None:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        try:
            delete_course_settings(CourseKey.from_string(course_id))
        except InvalidKeyError:
            data = {'message': 'Invalid Key'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)
        except NotFoundError:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        return APIResponse(success=True)


class UnitEnabledAPIView(APIView):
    """Handlers for module level settings"""

    def get(self, request, course_id=None, unit_id=None):
        """Gets the enabled state for a unit"""
        if course_id is None or unit_id is None:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        try:
            settings = get_unit_settings(
                CourseKey.from_string(course_id),
                UsageKey.from_string(unit_id),
            )
        except InvalidKeyError:
            data = {'message': 'Invalid Key'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)
        except NotFoundError:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        return APIResponse(success=True, data=settings)

    def post(self, request, course_id=None, unit_id=None):
        """Sets the enabled state for a unit"""

        enabled = request.data.get('enabled')

        try:
            set_unit_settings(
                CourseKey.from_string(course_id),
                UsageKey.from_string(unit_id),
                {'enabled': enabled})
        except InvalidKeyError:
            data = {'message': 'Invalid Key'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)
        except TypeError:
            data = {'message': 'Invalid parameters'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)

        return APIResponse(success=True)

    def delete(self, request, course_id=None, unit_id=None):
        """Deletes the settings for a unit"""

        if course_id is None or unit_id is None:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        try:
            delete_unit_settings(
                CourseKey.from_string(course_id),
                UsageKey.from_string(unit_id),
            )
        except InvalidKeyError:
            data = {'message': 'Invalid Key'}
            return APIResponse(http_status=status.HTTP_400_BAD_REQUEST, data=data)
        except NotFoundError:
            return APIResponse(http_status=status.HTTP_404_NOT_FOUND)

        return APIResponse(success=True)



# pyright: reportMissingImports=false

"""
Contain all imported functions coming out of the platform.

We know these functions will be available at run time, but they
cannot be imported normally.
"""


def get_text_transcript(video_block):
    """Get the transcript for a video block in text format, or None."""
    # pylint: disable=import-error, import-outside-toplevel
    from xmodule.exceptions import NotFoundError
    from xmodule.video_block.transcripts_utils import get_transcript
    try:
        transcript, _, _ = get_transcript(video_block, output_format='txt')
    except NotFoundError:
        # some old videos have no transcripts, just accept that reality
        return None
    return transcript


def get_block(usage_key):
    """Get a block from the module store given the usage key."""
    # pylint: disable=import-error, import-outside-toplevel
    from xmodule.modulestore.django import modulestore
    return modulestore().get_item(usage_key)



"""Tests for text utils used by the blocks"""
import unittest
from textwrap import dedent

from ai_aside.text_utils import html_to_text


class TestSummaryHookAside(unittest.TestCase):
    """Tests of text utils as used by the summary hook"""
    def test_html_to_text(self):
        html_content = '''\
            <div>
                <h1>Lorem Ipsum</h1>
                <p>Lorem ipsum dolor <em style="font-size: 99pt;">sit amet</em>, consectetur adipiscing elit.</p>
                <!-- The next paragraph will have an intentional mismatching close tag -->
                <p>Sed volutpat velit sed dui <i class="something">fringilla</i> fermentum.</div>
                <p>Nullam quis velit at turpis lacinia convallis.</p>
            </div>'''
        expected_text = dedent('''\
            Lorem Ipsum
            Lorem ipsum dolor sit amet, consectetur adipiscing elit.
            Sed volutpat velit sed dui fringilla fermentum.
            Nullam quis velit at turpis lacinia convallis.''')
        text = html_to_text(html_content)
        self.assertEqual(text, expected_text)

    def test_html_to_text_messy(self):
        html_content = '''\
            <custom>
                <mismatching>Lorem Ipsum</tags>
                > Lorem ipsum dolor sit amet, consectetur adipiscing elit.</p>
                > Sed volutpat velit sed dui fringilla fermentum.</42>
                <p>> Nullam quis velit at turpis lacinia convallis.</p>'''
        expected_text = dedent('''\
            Lorem Ipsum
            > Lorem ipsum dolor sit amet, consectetur adipiscing elit.
            > Sed volutpat velit sed dui fringilla fermentum.
            > Nullam quis velit at turpis lacinia convallis.''')
        text = html_to_text(html_content)
        self.assertEqual(text, expected_text)

    def test_html_to_text_iframe(self):
        html_content = '''\
            <iframe
                src="https://mitx.qualtrics.com/jfe/form/SV_8ocnXlmaDwMRFgF?user_id=%%USER_ID%%"
                height="1200"
                width="100%"
                title="Nothing useful can be parsed from this, maybe fetching the src in the future?"
            ></iframe>
            '''
        expected_text = dedent('')
        text = html_to_text(html_content)
        self.assertEqual(text, expected_text)


if __name__ == '__main__':
    unittest.main()



"""Tests for the models"""
from django.test import TestCase
from opaque_keys.edx.keys import CourseKey, UsageKey

from ai_aside.models import AIAsideCourseEnabled, AIAsideUnitEnabled

course_keys = [
    'course-v1:edX+DemoX+Demo_Course',
    'course-v1:edX+DemoX+Demo_Course-2',
]

unit_keys = [
    "block-v1:edX+DemoX+Demo_Course+type@vertical+block@vertical_0270f6de40fc",
    "block-v1:edX+DemoX+Demo_Course+type@vertical+block@vertical_321ac313f2de",
]


class TestModels(TestCase):
    """Utils model tests"""

    def setUp(self):
        super().setUp()

        # Create some test cases

        # First course is disabled
        AIAsideCourseEnabled.objects.create(
            course_key=CourseKey.from_string(course_keys[0]),
            enabled=False,
        )

        # 2nd course is enabled
        AIAsideCourseEnabled.objects.create(
            course_key=CourseKey.from_string(course_keys[1]),
            enabled=True,
        )

        # 2nd course has an enabled and a disabled unit
        AIAsideUnitEnabled.objects.create(
            course_key=CourseKey.from_string(course_keys[1]),
            unit_key=UsageKey.from_string(unit_keys[0]),
            enabled=True,
        )

        AIAsideUnitEnabled.objects.create(
            course_key=CourseKey.from_string(course_keys[1]),
            unit_key=UsageKey.from_string(unit_keys[1]),
            enabled=False,
        )

    def test_course_enabled(self):
        """Check that the models work as expected."""

        self.assertEqual(AIAsideCourseEnabled.objects.count(), 2)

        self.assertFalse(AIAsideCourseEnabled.objects.filter(
            course_key=course_keys[0],
        ).first().enabled)

        self.assertTrue(AIAsideCourseEnabled.objects.filter(
            course_key=course_keys[1],
        ).first().enabled)

    def test_unit_enabled(self):
        """Check that the models work as expected."""

        self.assertEqual(AIAsideUnitEnabled.objects.count(), 2)

        self.assertTrue(AIAsideUnitEnabled.objects.filter(
            course_key=course_keys[1],
            unit_key=unit_keys[0],
        ).last().enabled)

        self.assertFalse(AIAsideUnitEnabled.objects.filter(
            course_key=course_keys[1],
            unit_key=unit_keys[1],
        ).last().enabled)
